#include "board.h"
#include "constants.h"
#include "tables.h"
#include "game.h"

//-------------------------------------------------------------------------

Board :: Board()
{
	for ( int x = 0; x <= 7; x++ )
		for ( int y = 0; y <= 7; y++ )
			boardArray[x][y] = NULL;
	whiteKing = NULL;
	blackKing = NULL;
}

void Board :: addPiece( int x, int y, int id )
{
	boardArray[x][y] = new Piece(id, x, y);
	if ( boardArray[x][y] -> type() == KING )
		if ( boardArray[x][y] -> color() == WHITE )
			whiteKing = boardArray[x][y];
		else 
			blackKing = boardArray[x][y];
}

void Board :: addPiece( int x, int y, int color, int type )
{
	addPiece( x, y, color | type );
}

void Board :: makeMove( Move move )
{
	Coordinates from = move.from;
	Coordinates to = move.to;
	boardArray[to.x][to.y] = boardArray[from.x][from.y];
	boardArray[to.x][to.y] -> x = to.x;
	boardArray[to.x][to.y] -> y = to.y;
	boardArray[from.x][from.y] = NULL;

	if ( boardArray[to.x][to.y] -> type() == KING )// 
	{
		if ( from.x - to.x == 2 ) //.     
		{
			boardArray[3][to.y] = boardArray[0][to.y];
			boardArray[3][to.y] -> x = 3;
			boardArray[3][to.y] -> moved = true;
			boardArray[0][to.y] = NULL;
		}
		if ( to.x - from.x == 2 ) //.     
		{
			boardArray[5][to.y] = boardArray[7][to.y];
			boardArray[5][to.y] -> x = 5;
			boardArray[5][to.y] -> moved = true;
			boardArray[7][to.y] = NULL;
		}
	}

	if ( move.isEnpassant )
	{
		if ( boardArray[to.x][to.y] -> color() == BLACK )
			boardArray[to.x][to.y - 1] = NULL;
		if ( boardArray[to.x][to.y] -> color() == WHITE )
			boardArray[to.x][to.y + 1] = NULL;
	}

	game.enpassant.x = -1;
	game.enpassant.y = -1;

	if ( boardArray[to.x][to.y] -> type() == PAWN ) //   
	{
		if ( boardArray[to.x][to.y] -> color() == WHITE && from.y - to.y == 2 ) //  
		{
			game.enpassant.x = to.x;
			game.enpassant.y = to.y + 1;
		}
		if ( boardArray[to.x][to.y] -> color() == BLACK && to.y - from.y == 2 ) //  
		{
			game.enpassant.x = to.x;
			game.enpassant.y = to.y - 1;
		}
	}

	boardArray[to.x][to.y] -> moved = true;

	if ( move.isPromotion )
	{
		changeScore( -pieceIdToValue[ boardArray[to.x][to.y] -> id ] );
		changeScore( pieceIdToValue[ move.promotionId ] );
		boardArray[to.x][to.y] -> id = move.promotionId;
	}

	if ( move.captured )
	{
		changeScore( -pieceIdToValue[ ((Piece*)(move.captured)) -> id ] );
	}

	game.changeTurn();
}

void Board :: cancelMove( Move move )
{
	Coordinates from = move.from;
	Coordinates to = move.to;
	boardArray[from.x][from.y] = boardArray[to.x][to.y];
	boardArray[from.x][from.y] -> x = from.x;
	boardArray[from.x][from.y] -> y = from.y;
	if ( !move.isEnpassant )
		boardArray[to.x][to.y] = (Piece*)move.captured;
	else
		boardArray[to.x][to.y] = NULL;

	if ( boardArray[from.x][from.y] -> type() == KING )// 
	{
		if ( from.x - to.x == 2 ) //.     
		{
			boardArray[0][to.y] = boardArray[3][to.y];
			boardArray[0][to.y] -> x = 0;
			boardArray[0][to.y] -> moved = false;
			boardArray[3][to.y] = NULL;
		}
		if ( to.x - from.x == 2 ) //.     
		{
			boardArray[7][to.y] = boardArray[5][to.y];
			boardArray[7][to.y] -> x = 7;
			boardArray[7][to.y] -> moved = false;
			boardArray[5][to.y] = NULL;
		}
	}

	game.enpassant = move.savedEnpassant;

	if ( move.isEnpassant )
	{
		if ( boardArray[from.x][from.y] -> color() == BLACK )
			boardArray[to.x][to.y - 1] = (Piece*)move.captured;
		if ( boardArray[from.x][from.y] -> color() == WHITE )
			boardArray[to.x][to.y + 1] = (Piece*)move.captured;
	}

	boardArray[from.x][from.y] -> moved = move.movedBefore;

	if ( move.isPromotion )
	{
		changeScore( -pieceIdToValue[ boardArray[from.x][from.y] -> id ] );
		boardArray[from.x][from.y] -> id = boardArray[from.x][from.y] -> color() | PAWN;
		changeScore( pieceIdToValue[ boardArray[from.x][from.y] -> id ] );
	}

	if ( move.captured )
	{
		changeScore( pieceIdToValue[ ((Piece*)(move.captured)) -> id ] );// 
	}

	game.changeTurn();
}

bool Board :: isCheck( int color )
{
	if ( color == WHITE )
		return whiteKing -> isAttacked();
	else
		return blackKing -> isAttacked();
}

void Board :: pasteFen( string fen )
{
	clear();
	int x = 0, y = 0;
	for ( unsigned int i = 0; i < fen.size(); i++ )
		if ( fen[i] >= '0' && fen[i] <= '9' )            
		{
			int number = fen[i] - '0';
			x += number;
		} 
		else if ( fen[i] == '/' )
		{
			y++;
			x = 0;
		} 
		else
		{
			int id = charToPieceIdTable[fen[i]];
			addPiece( x, y, id );
			x++;
		}
}

string Board :: toFen()
{
	string result;
	for ( int y = 0; y <= 7; y++ )
	{
		int space = 0;
		for ( int x = 0; x <= 7; x++ )
		{
			if (boardArray[x][y])
			{
				if ( space > 0 )
					result += space + '0';
				space = 0;
				result += boardArray[x][y] -> toChar();
			}
			else
			{
				space++;
			}
		}
		if ( space > 0 )
			result += space + '0';
		if (y < 7)
			result += '/';
	}
	return result;
}

void Board :: startPos() 
{
	pasteFen( "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR" );
}

void Board :: clear()
{
	for ( int x = 0; x <= 7; x++ )
		for ( int y = 0; y <= 7; y++ )
		{
			delete boardArray[x][y];
			boardArray[x][y] = NULL;
		}
	whiteKing = NULL;
	blackKing = NULL;
}

string Board :: toString()
{
	string result;
	for ( int y = 0; y <= 7; y++ )
	{
		for ( int x = 0; x <= 7; x++ )
		{
			if ( boardArray[x][y] )
				result += boardArray[x][y] -> toChar();
			else
				result += '.';
		}
		result += '\n';
	} 
	return result;
}

void Board :: setCastlingWhite( bool longc, bool shortc )
{
	if (longc)
		boardArray[0][7] -> moved = false;
	if (shortc)
		boardArray[7][7] -> moved = false;
	if (shortc || longc)
		boardArray[4][7] -> moved = false;
}

void Board :: setCastlingBlack( bool longc, bool shortc )
{
	if (longc)
		boardArray[0][0] -> moved = false;
	if (shortc)
		boardArray[7][0] -> moved = false;
	if (shortc || longc)
		boardArray[4][0] -> moved = false;
}

void Board :: parseCastling( string castling )
{
	for ( unsigned int i = 0; i < castling.size(); i++ )
	{
		if ( castling[i] == 'K' ) setCastlingWhite( false, true );
		if ( castling[i] == 'Q' ) setCastlingWhite( true, false );
		if ( castling[i] == 'k' ) setCastlingBlack( false, true );
		if ( castling[i] == 'q' ) setCastlingBlack( true, false );
	}
}