#include "game.h"
#include "constants.h"
#include "tables.h"
#include "evaluation.h"
#include <sstream>

//-------------------------------------------------------------------------

Game game;

//-------------------------------------------------------------------------

void Game :: newGame()
{
	initiateTables();
	board.startPos();
	insignificantPlies = 0;
	turn = WHITE;
	totalMoves = 1;
	board.parseCastling( "KQkq" );
	evaluateInit();
}

void Game :: gameFromFen( string fen )
{
	initiateTables();
	stringstream fenstream;
	fenstream << fen;

	string position;
	fenstream >> position;
	board.pasteFen( position );

	string turnColor;
	fenstream >> turnColor;
	if ( turnColor == "w" )
		turn = WHITE;
	if ( turnColor == "b" )
		turn = BLACK;

	string castling;
	fenstream >> castling;
	board.parseCastling( castling );

	string enPassant;
	fenstream >> enPassant;
	if ( enPassant == "-" )
		game.enpassant = Coordinates( -1, -1 );
	else
		game.enpassant = Coordinates( enPassant );

	fenstream >> insignificantPlies;
	fenstream >> totalMoves;
	evaluateInit();
}

list<Move> sortMoves( list<Move> input )
{
	list<Move> result;

	//TODO: standart sort
	for ( list<Move>::iterator move = input.begin(); move != input.end(); move++ )
	{
		if ( move -> captured )
			result.push_front( *move );
		else
			result.push_back( *move );
	}

	return result;
}

list<Move> Game::generateMoves( Move* pvMove )
{
	list<Move> result;

	for ( int x = 0; x <= 7; x++ )
		for ( int y = 0; y <= 7; y++ )
		{
			if ( board.boardArray[x][y] && board.boardArray[x][y] -> color() == turn )
			{
				list<Move> moves = board.boardArray[x][y] -> generateMoves();

				for ( list<Move>::iterator move = moves.begin(); move != moves.end(); move++ )
					if ( move -> captured || move -> isPromotion
							|| ( pvMove && move -> toString() == pvMove -> toString() ) )
						result.push_front( *move );
					else
						result.push_back( *move );
			}
		}
	
	return result;//sortMoves(result);
}

void Game :: parseMoves( string moves )
{
	stringstream movesStream;
	movesStream << moves;
	while ( !movesStream.eof() )
	{
		string moveString;
		movesStream >> moveString;
		Move move( moveString );
		game.board.makeMove( move );
		totalMoves++;
	}
}

void Game :: parseMovesCancel( string moves )
{
	stringstream movesStream;
	movesStream << moves;
	while ( !movesStream.eof() )
	{
		string moveString;
		movesStream >> moveString;
		Move move( moveString );
		game.board.cancelMove( move );
	}
}

void Game::changeTurn()
{
	if ( turn == WHITE ) 
		turn = BLACK;
	else if ( turn == BLACK ) 
		turn = WHITE;
}
