#include "moves.h"
#include "game.h"
#include "constants.h"

//-------------------------------------------------------------------------

Move :: Move( int fromX, int fromY, int toX, int toY, int promotionType )
{
	from = Coordinates( fromX, fromY );
	to = Coordinates( toX, toY );
	captured = game.board.boardArray[toX][toY];

	movedBefore = game.board.boardArray[fromX][fromY] -> moved;

	if ( toX == game.enpassant.x && toY == game.enpassant.y && game.board.boardArray[fromX][fromY] -> type() == PAWN )
		isEnpassant = true;
	else
		isEnpassant = false;

	promotionId = promotionType & TYPE_MASK | game.board.boardArray[from.x][from.y] -> color();
	isPromotion = ( promotionType != 0 );
	savedEnpassant = game.enpassant;

	if ( isEnpassant )
	{
		if ( game.board.boardArray[fromX][fromY]->color() == WHITE )
			captured = game.board.boardArray[game.enpassant.x][game.enpassant.y + 1];
		if ( game.board.boardArray[fromX][fromY]->color() == BLACK )
			captured = game.board.boardArray[game.enpassant.x][game.enpassant.y - 1];
	}
}

Move :: Move( Coordinates from, Coordinates to, int promotionType )
{
	this -> from = from;
	this -> to = to;
	captured = game.board.boardArray[to.x][to.y];

	movedBefore = game.board.boardArray[from.x][from.y] -> moved;

	promotionId = promotionType & TYPE_MASK | game.board.boardArray[from.x][from.y] -> color();
	isPromotion = ( promotionType != 0 );

	if ( to.x == game.enpassant.x && to.y == game.enpassant.y && game.board.boardArray[from.x][from.y] -> type() == PAWN )
		isEnpassant = true;
	else
		isEnpassant = false;
	savedEnpassant = game.enpassant;

	if ( isEnpassant )
	{
		if ( game.board.boardArray[from.x][from.y]->color() == WHITE )
			captured = game.board.boardArray[game.enpassant.x][game.enpassant.y + 1];
		if ( game.board.boardArray[from.x][from.y]->color() == BLACK )
			captured = game.board.boardArray[game.enpassant.x][game.enpassant.y - 1];
	}
}

Move :: Move( string moveString )
{
	int fromX = charToIntCoordinateX[ moveString[0] ];
	int fromY = charToIntCoordinateY[ moveString[1] ];
	int toX = charToIntCoordinateX[ moveString[2] ];
	int toY = charToIntCoordinateY[ moveString[3] ];
	this -> from = Coordinates( fromX, fromY );
	this -> to = Coordinates( toX, toY );

	captured = game.board.boardArray[to.x][to.y];

	movedBefore = game.board.boardArray[from.x][from.y] -> moved;

	if (moveString.size() > 4)
	{
		int promotionType = charToPieceIdTable[ moveString[4] ] & TYPE_MASK;
		promotionId = promotionType | game.board.boardArray[from.x][from.y] -> color();
		isPromotion = true;
	}
	else
		isPromotion = false;

	if ( toX == game.enpassant.x && toY == game.enpassant.y && game.board.boardArray[fromX][fromY] -> type() == PAWN )
		isEnpassant = true;
	else
		isEnpassant = false;
	savedEnpassant = game.enpassant;

	if ( isEnpassant )
	{
		if ( game.board.boardArray[fromX][fromY]->color() == WHITE )
			captured = game.board.boardArray[game.enpassant.x][game.enpassant.y + 1];
		if ( game.board.boardArray[fromX][fromY]->color() == BLACK )
			captured = game.board.boardArray[game.enpassant.x][game.enpassant.y - 1];
	}
}

/*string Move :: toString()
{
	string type = "-";
	string piece = "";
	string promotionPiece = "";
	string enpassant = "";

	if ( game.board.boardArray[from.x][from.y] -> type() == KING ) // 
	{
		if ( from.x - to.x == 2 ) //
			return "O-O-O";
		if ( to.x - from.x == 2 ) //
			return "O-O";
	}

	if ( captured != 0 || isEnpassant ) 
		type = "x";

	if ( isEnpassant ) 
		enpassant = "/ep";

	piece = toupper(game.board.boardArray[from.x][from.y] -> toChar());

	if ( isPromotion )
		promotionPiece = toupper(pieceIdToCharTable[promotionId]);

	if (piece == "P")
		return from.toString() + type + to.toString() + promotionPiece + enpassant;

	return piece + from.toString() + type + to.toString();
}*/

string Move :: toString()
{
	string promotionPiece = "";
	if ( isPromotion )
		promotionPiece = toupper(pieceIdToCharTable[promotionId]);
	return from.toString() + "-" + to.toString() + promotionPiece;
}