#include "pieces.h"
#include "tables.h"
#include "constants.h"
#include "game.h"
#include <iostream>

//-------------------------------------------------------------------------

Piece :: Piece( int id, int x, int y )
{
	this -> id = id;
	this -> x = x;
	this -> y = y;
	moved = true;
}

Piece :: Piece( int color, int type, int x, int y )
{
	this -> id = color | type;
	this -> x = x;
	this -> y = y;
	moved = true;
}

int Piece :: type()
{
	return id & TYPE_MASK;
}

int Piece :: color()
{
	return id & COLOR_MASK;
}

char Piece :: toChar()
{
	return pieceIdToCharTable[ id ];
}

bool Piece::isAttacked() //INFO:  .    
{              
	unsigned int nx;
	unsigned int ny;

	//-------Knight--Moves

	nx = x + 1;  
	ny = y + 2;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	nx = x - 1;  
	ny = y + 2;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	nx = x + 1;  
	ny = y - 2;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	nx = x - 1;  
	ny = y - 2;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	nx = x + 2;  
	ny = y + 1;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	nx = x - 2;  
	ny = y + 1;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	nx = x + 2;  
	ny = y - 1;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	nx = x - 2;  
	ny = y - 1;  

	if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() != this -> color() &&
		game.board.boardArray[nx][ny] -> type() == KNIGHT )
	{
		return true;
	}

	//---End-Knight--Moves

	//---Rook--Moves

	for ( nx = x + 1; nx <= 7; nx++ )
	{
		if ( game.board.boardArray[nx][y] )
		{
			if ( game.board.boardArray[nx][y] -> color() != this -> color() &&
				( game.board.boardArray[nx][y] -> type() == ROOK || game.board.boardArray[nx][y] -> type() == QUEEN ) )
				return true;
			break;
		}
	}

	for ( nx = x - 1; nx >= 0 && nx <= 7; nx-- )
	{
		if ( game.board.boardArray[nx][y] )
		{
			if ( game.board.boardArray[nx][y] -> color() != this -> color() &&
				( game.board.boardArray[nx][y] -> type() == ROOK || game.board.boardArray[nx][y] -> type() == QUEEN ) )
				return true;
			break;
		}
	}

	for ( ny = y + 1; ny <= 7; ny++ )
	{
		if ( game.board.boardArray[x][ny] )
		{
			if ( game.board.boardArray[x][ny] -> color() != this -> color() &&
				( game.board.boardArray[x][ny] -> type() == ROOK || game.board.boardArray[x][ny] -> type() == QUEEN ) )
				return true;
			break;
		}
	}

	for ( ny = y - 1; ny >= 0 && ny <= 7; ny-- )
	{
		if ( game.board.boardArray[x][ny] )
		{
			if ( game.board.boardArray[x][ny] -> color() != this -> color() &&
				( game.board.boardArray[x][ny] -> type() == ROOK || game.board.boardArray[x][ny] -> type() == QUEEN ) )
				return true;
			break;
		}
	}

	//---End-Rook--Moves

	//---Bishop--Moves

	nx = x + 1;
	ny = y + 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
				( game.board.boardArray[nx][ny] -> type() == BISHOP || game.board.boardArray[nx][ny] -> type() == QUEEN ) )
				return true;
			break;
		}
		nx++;
		ny++;
	}

	nx = x + 1;
	ny = y - 1;

	for ( ; nx <= 7 && ny >= 0 && nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
				( game.board.boardArray[nx][ny] -> type() == BISHOP || game.board.boardArray[nx][ny] -> type() == QUEEN ) )
				return true;
			break;
		}
		nx++;
		ny--;
	}

	nx = x - 1;
	ny = y + 1;

	for ( ; nx >= 0 && ny <= 7 && nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
				( game.board.boardArray[nx][ny] -> type() == BISHOP || game.board.boardArray[nx][ny] -> type() == QUEEN ) )
				return true;
			break;
		}
		nx--;
		ny++;
	}

	nx = x - 1;
	ny = y - 1;

	for ( ; nx >= 0 && ny >= 0 && nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
				( game.board.boardArray[nx][ny] -> type() == BISHOP || game.board.boardArray[nx][ny] -> type() == QUEEN ) )
				return true;
			break;
		}
		nx--;
		ny--;
	}

	//---End-Bishop--Moves

	//---Pawn--Moves

	if ( color() == WHITE )
	{
		nx = x + 1;
		ny = y - 1;
		if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == PAWN )
				return true;
		}
		nx = x - 1;
		ny = y - 1;
		if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == PAWN )
				return true;
		}
	}
	else
		if ( color() == BLACK )
		{
			nx = x + 1;
			ny = y + 1;
			if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] )
			{
				if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
					game.board.boardArray[nx][ny] -> type() == PAWN )
					return true;
			}
			nx = x - 1;
			ny = y + 1;
			if ( nx <= 7 && ny <= 7 && game.board.boardArray[nx][ny] )
			{
				if ( game.board.boardArray[nx][ny] -> color() != this -> color() &&
					game.board.boardArray[nx][ny] -> type() == PAWN )
					return true;
			}
		}

		//---End-Pawn--Moves

		//---King--Moves

		nx = x + 1;
		ny = y + 1;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		nx = x + 1;
		ny = y;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		nx = x + 1;
		ny = y - 1;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		nx = x;
		ny = y + 1;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		nx = x;
		ny = y - 1;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		nx = x - 1;
		ny = y + 1;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		nx = x - 1;
		ny = y;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		nx = x - 1;
		ny = y - 1;

		if ( nx <= 7 && ny <= 7 )
		{
			if ( game.board.boardArray[nx][ny] &&
				game.board.boardArray[nx][ny] -> color() != this -> color() &&
				game.board.boardArray[nx][ny] -> type() == KING )
				return true;
		}

		//---End-King--Moves

		return false; 
}

//---MOVES-GENERATOR-------------------------------------------------------

bool isAvailibleKingMove( int x, int y, unsigned int nx, unsigned int ny, int c )
{
	if ( nx == x && ny == y ) return false;
	if ( nx > 7 || ny > 7 ) return false;
	if ( game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() == c ) return false;

	bool result = true;
	Move move( x, y, nx, ny );
	game.board.makeMove( move );
	if ( game.board.isCheck( c ) ) result = false;
	game.board.cancelMove( move );

	return result;
}

bool isAvailibleRookMove( int x, int y, unsigned int nx, unsigned int ny, int c )
{
	if ( nx == x && ny == y ) return false;
	if ( nx > 7 || ny > 7 ) return false;

	if ( nx != x && ny != y ) return false; //  

	if ( game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() == c ) return false; //  

	bool result = true;
	Move move( x, y, nx, ny );
	game.board.makeMove( move );
	if ( game.board.isCheck( c ) ) result = false;
	game.board.cancelMove( move );

	return result;
}

bool isAvailibleBishopMove( int x, int y, unsigned int nx, unsigned int ny, int c )
{
	if ( nx == x && ny == y ) return false;
	if ( nx > 7 || ny > 7 ) return false;

	if ( game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() == c ) return false; //  

	bool result = true;
	Move move( x, y, nx, ny );
	game.board.makeMove( move );
	if ( game.board.isCheck( c ) ) result = false;
	game.board.cancelMove( move );

	return result;
}


bool isAvailibleKnightMove( int x, int y, unsigned int nx, unsigned int ny, int c )
{
	if ( nx == x || ny == y ) return false;
	if ( nx > 7 || ny > 7 ) return false;

	if ( abs(int(x - nx)) == abs(int(y - ny)) ) return false;

	if ( game.board.boardArray[nx][ny] && 
		game.board.boardArray[nx][ny] -> color() == c ) return false;

	bool result = true;
	Move move( x, y, nx, ny );
	game.board.makeMove( move );
	if ( game.board.isCheck( c ) ) result = false;
	game.board.cancelMove( move );

	return result;
}

bool isAvailibleBlackPawnMove( int x, int y, unsigned int nx, unsigned int ny )
{
	if ( nx == x && ny == y ) return false;
	if ( nx > 7 || ny > 7 ) return false;

	if ( nx == x ) // 
	{
		if ( game.board.boardArray[nx][ny] ) //    
			return false;
		if ( y == 1 && ny == 3 && game.board.boardArray[nx][2] ) // 
			return false;
		if ( y != 1 && ny - y >= 2 ) //   2       
			return false;
	}

	if ( nx != x ) // 
	{
		if ( nx - x != 1 && x - nx != 1 ) //      
			return false;
		if ( ny - y != 1 ) //     
			return false;
		if ( !game.board.boardArray[nx][ny] && !( nx == game.enpassant.x && ny == game.enpassant.y ) ) //    )
			return false;
		if ( !( nx == game.enpassant.x && ny == game.enpassant.y ) && (!game.board.boardArray[nx][ny] || game.board.boardArray[nx][ny] -> color() != WHITE) ) //   
			return false;
		if ( !game.board.boardArray[nx][3] && (nx == game.enpassant.x && ny == game.enpassant.y
			&& game.board.boardArray[nx][3] && game.board.boardArray[nx][3] -> color() == BLACK) )
			return false;
	}

	bool result = true;
	Move move( x, y, nx, ny );
	game.board.makeMove( move );
	if ( game.board.isCheck( BLACK ) ) result = false;
	game.board.cancelMove( move );

	return result;
}

bool isAvailibleWhitePawnMove( int x, int y, unsigned int nx, unsigned int ny )
{
	if ( nx == x && ny == y ) return false;
	if ( nx > 7 || ny > 7 ) return false;

	if ( nx == x ) // 
	{
		if ( game.board.boardArray[nx][ny] ) //    
			return false;
		if ( y == 6 && ny == 4 && game.board.boardArray[nx][5] ) // 
			return false;
		if ( y != 6 && y - ny >= 2 ) //   2       
			return false;
	}

	if ( nx != x ) // 
	{
		if ( nx - x != 1 && x - nx != 1 ) //      
			return false;
		if ( y - ny != 1 ) //     
			return false;
		if ( !game.board.boardArray[nx][ny] && !( nx == game.enpassant.x && ny == game.enpassant.y ) ) //    )
			return false;
		if ( !( nx == game.enpassant.x && ny == game.enpassant.y ) && (!game.board.boardArray[nx][ny] || game.board.boardArray[nx][ny] -> color() != BLACK) ) //   
			return false;
		if ( !game.board.boardArray[nx][5] && (nx == game.enpassant.x && ny == game.enpassant.y 
			&& game.board.boardArray[nx][5] && game.board.boardArray[nx][5] -> color() == WHITE) )
			return false;
	}

	bool result = true;
	Move move( x, y, nx, ny );
	game.board.makeMove( move );
	if ( game.board.isCheck( WHITE ) ) result = false;
	game.board.cancelMove( move );

	return result;
}

list<Move> Piece :: generateKingMoves()
{
	list<Move> result;

	for ( int nx = x - 1; nx <= x + 1; nx++ )
		for ( int ny = y - 1; ny <= y + 1; ny++ )
			if ( isAvailibleKingMove( x, y, nx, ny, color() ) )
			{
				result.push_back( Move( x, y, nx, ny ) );
			}

	if ( !moved && !isAttacked() ) //      -  
	{
		if ( game.board.boardArray[0][y] && game.board.boardArray[0][y] -> type() == ROOK
			&& !game.board.boardArray[0][y] -> moved && x == 4
			&& !game.board.boardArray[1][y] && !game.board.boardArray[2][y]
		    && !game.board.boardArray[3][y]) // 
		{
			Move move = Move( x, y, 2, y );
			game.board.makeMove( move );
			if ( !isAttacked() && !game.board.boardArray[3][y] -> isAttacked() )
				result.push_back( move );
			game.board.cancelMove( move );
		}

		if ( game.board.boardArray[7][y] && game.board.boardArray[7][y] -> type() == ROOK
			&& !game.board.boardArray[7][y] -> moved && x == 4
			&& !game.board.boardArray[5][y] && !game.board.boardArray[6][y]) // 
		{
			Move move = Move( x, y, 6, y );
			game.board.makeMove( move );
			if ( !isAttacked() && !game.board.boardArray[5][y] -> isAttacked() )
				result.push_back( move );
			game.board.cancelMove( move );
		}
	}
			
	return result;
}

list<Move> Piece :: generateQueenMoves()
{
	list<Move> result;

	for ( int nx = x + 1; nx <= 7; nx++ ) //
	{
		if ( isAvailibleRookMove( x, y, nx, y, color() ) )
		{
			result.push_back( Move( x, y, nx, y ) );
		}
		if ( game.board.boardArray[nx][y] ) break;
	}

	for ( int nx = x - 1; nx <= 7; nx-- ) //
	{
		if ( isAvailibleRookMove( x, y, nx, y, color() ) )
		{
			result.push_back( Move( x, y, nx, y ) );
		}
		if ( game.board.boardArray[nx][y] ) break;
	}

	for ( int ny = y + 1; ny <= 7; ny++ ) //
	{
		if ( isAvailibleRookMove( x, y, x, ny, color() ) )
		{
			result.push_back( Move( x, y, x, ny ) );
		}
		if ( game.board.boardArray[x][ny] ) break;
	}

	for ( int ny = y - 1; ny <= 7; ny-- ) //
	{
		if ( isAvailibleRookMove( x, y, x, ny, color() ) )
		{
			result.push_back( Move( x, y, x, ny ) );
		}
		if ( game.board.boardArray[x][ny] ) break;
	}

	unsigned int nx = x + 1;
	unsigned int ny = y + 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color()
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx++;
		ny++;
	}

	nx = x + 1;
	ny = y - 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color()
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx++;
		ny--;
	}

	nx = x - 1;
	ny = y + 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color()
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx--;
		ny++;
	}

	nx = x - 1;
	ny = y - 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() 
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx--;
		ny--;
	}

	return result;
}


list<Move> Piece::generateRookMoves()
{
	list<Move> result;

	for ( int nx = x + 1; nx <= 7; nx++ ) //
	{
		if ( isAvailibleRookMove( x, y, nx, y, color() ) )
		{
			result.push_back( Move( x, y, nx, y ) );
		}
		if ( game.board.boardArray[nx][y] ) break;
	}

	for ( int nx = x - 1; nx <= 7; nx-- ) //
	{
		if ( isAvailibleRookMove( x, y, nx, y, color() ) )
		{
			result.push_back( Move( x, y, nx, y ) );
		}
		if ( game.board.boardArray[nx][y] ) break;
	}

	for ( int ny = y + 1; ny <= 7; ny++ ) //
	{
		if ( isAvailibleRookMove( x, y, x, ny, color() ) )
		{
			result.push_back( Move( x, y, x, ny ) );
		}
		if ( game.board.boardArray[x][ny] ) break;
	}

	for ( int ny = y - 1; ny <= 7; ny-- ) //
	{
		if ( isAvailibleRookMove( x, y, x, ny, color() ) )
		{
			result.push_back( Move( x, y, x, ny ) );
		}
		if ( game.board.boardArray[x][ny] ) break;
	}

	return result;
}

list<Move> Piece :: generateBishopMoves()
{
	list<Move> result;

	unsigned int nx = x + 1;
	unsigned int ny = y + 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color()
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx++;
		ny++;
	}

	nx = x + 1;
	ny = y - 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color()
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx++;
		ny--;
	}

	nx = x - 1;
	ny = y + 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color()
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx--;
		ny++;
	}

	nx = x - 1;
	ny = y - 1;

	for ( ; nx <= 7 && ny <= 7; )
	{
		if ( game.board.boardArray[nx][ny] )
		{
			if ( game.board.boardArray[nx][ny] -> color() != this -> color() 
				&& isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
			break;
		}
		else
		{
			if ( isAvailibleBishopMove( x, y, nx, ny, color() ) )
				result.push_back( Move( x, y, nx, ny ) );
		}
		nx--;
		ny--;
	}

	return result;
}


list<Move> Piece :: generateKnightMoves()
{
	list<Move> result;

	for ( int nx = x - 2; nx <= x + 2; nx++ )
		for ( int ny = y - 2; ny <= y + 2; ny++ )
			if ( isAvailibleKnightMove( x, y, nx, ny, color() ) )
			{
				result.push_back( Move( x, y, nx, ny ) );
			}

			return result;
}

list<Move> Piece :: generateBlackPawnMoves()
{
	list<Move> result;

	// 
	for ( int ny = y + 1; ny <= y + 2; ny++ )
	{
		if ( isAvailibleBlackPawnMove( x, y, x, ny ) )
		{
			//
			if ( ny == 7 )
			{
				result.push_back( Move( x, y, x, ny, WHITE | QUEEN ) );
				result.push_back( Move( x, y, x, ny, WHITE | ROOK ) );
				result.push_back( Move( x, y, x, ny, WHITE | BISHOP ) );
				result.push_back( Move( x, y, x, ny, WHITE | KNIGHT ) );
			}
			else
			{
				result.push_back( Move( x, y, x, ny ) );
			}
		}
	}

	//
	int nx = x - 1;
	{
		if ( isAvailibleBlackPawnMove( x, y, nx, y + 1 ) )
		{
			if ( y == 6 )
			{
				result.push_back( Move( x, y, nx, y + 1, WHITE | QUEEN ) );
				result.push_back( Move( x, y, nx, y + 1, WHITE | ROOK ) );
				result.push_back( Move( x, y, nx, y + 1, WHITE | BISHOP ) );
				result.push_back( Move( x, y, nx, y + 1, WHITE | KNIGHT ) );
			}
			else
			{
				result.push_back( Move( x, y, nx, y + 1 ) );
			}
		}
	}

	nx = x + 1;
	{
		if ( isAvailibleBlackPawnMove( x, y, nx, y + 1 ) )
		{
			if ( y == 6 )
			{
				result.push_back( Move( x, y, nx, y + 1, WHITE | QUEEN ) );
				result.push_back( Move( x, y, nx, y + 1, WHITE | ROOK ) );
				result.push_back( Move( x, y, nx, y + 1, WHITE | BISHOP ) );
				result.push_back( Move( x, y, nx, y + 1, WHITE | KNIGHT ) );
			}
			else
			{
				result.push_back( Move( x, y, nx, y + 1 ) );
			}
		}
	}

	return result;
}

list<Move> Piece :: generateWhitePawnMoves()
{
	list<Move> result;

	// 
	for ( int ny = y - 1; ny >= y - 2; ny-- )
	{
		if ( isAvailibleWhitePawnMove( x, y, x, ny ) )
		{
			//
			if ( ny == 0 )
			{
				result.push_back( Move( x, y, x, ny, WHITE | QUEEN ) );
				result.push_back( Move( x, y, x, ny, WHITE | ROOK ) );
				result.push_back( Move( x, y, x, ny, WHITE | BISHOP ) );
				result.push_back( Move( x, y, x, ny, WHITE | KNIGHT ) );
			}
			else
			{
				result.push_back( Move( x, y, x, ny ) );
			}
		}
	}

	//
	int nx = x - 1;
	{
		if ( isAvailibleWhitePawnMove( x, y, nx, y - 1 ) )
		{
			if ( y == 1 )
			{
				result.push_back( Move( x, y, nx, y - 1, WHITE | QUEEN ) );
				result.push_back( Move( x, y, nx, y - 1, WHITE | ROOK ) );
				result.push_back( Move( x, y, nx, y - 1, WHITE | BISHOP ) );
				result.push_back( Move( x, y, nx, y - 1, WHITE | KNIGHT ) );
			}
			else
			{
				result.push_back( Move( x, y, nx, y - 1 ) );
			}
		}
	}

	nx = x + 1;
	{
		if ( isAvailibleWhitePawnMove( x, y, nx, y - 1 ) )
		{
			if ( y == 1 )
			{
				result.push_back( Move( x, y, nx, y - 1, WHITE | QUEEN ) );
				result.push_back( Move( x, y, nx, y - 1, WHITE | ROOK ) );
				result.push_back( Move( x, y, nx, y - 1, WHITE | BISHOP ) );
				result.push_back( Move( x, y, nx, y - 1, WHITE | KNIGHT ) );
			}
			else
			{
				result.push_back( Move( x, y, nx, y - 1 ) );
			}
		}
	}

	return result;
}

list<Move> Piece :: generateMoves()
{
	if ( type() == KING )
		return generateKingMoves();
	else if ( type() == QUEEN )
		return generateQueenMoves();
	else if ( type() == ROOK )
		return generateRookMoves();
	else if ( type() == BISHOP )
		return generateBishopMoves();
	else if ( type() == KNIGHT )
		return generateKnightMoves();
	else if ( type() == PAWN && color() == WHITE )
		return generateWhitePawnMoves();
	else if ( type() == PAWN && color() == BLACK )
		return generateBlackPawnMoves();
	return *((list<Move>*)0);
}