#include "search.h"
#include "constants.h"
#include "moves.h"
#include "game.h"
#include "evaluation.h"
#include "assert.h"
#include <iostream>
#include "pv.h"
#include <sstream>
#include <windows.h>
#include <conio.h>


using std :: cout;
using std :: cin;
using std :: endl;
using std :: stringstream;

int nodes = 0;
int maxScore = 0;
int needDepth;
int stopTime;

bool stopSearch;

unsigned int get_ms()
{
	timeb time_buffer;
	ftime(&time_buffer);
	return (unsigned int(time_buffer.time * 1000 + time_buffer.millitm));
}

int forced( int alpha, int beta, int maxCheckDepth )
{
	nodes++;
	int score = evaluate();

	if ( maxCheckDepth <= 0 ) return score;

	if ( score >= beta ) return beta;
	if ( score > alpha ) alpha = score;

	//TODO: Captures generator
	list<Move> moves = game.generateMoves();

	for ( list<Move>::iterator move = moves.begin(); move != moves.end(); move++ )
	{
		if ( move -> captured )
		{
			game.board.makeMove( *move );
			alpha = -forced( -beta, -alpha, maxCheckDepth );
			game.board.cancelMove( *move );
			if ( alpha >= beta ) return alpha;
		}
		else
			if ( game.board.isCheck( game.turn ) )
			{
				game.board.makeMove( *move );
				alpha = -forced( -beta, -alpha, maxCheckDepth - 1 );
				game.board.cancelMove( *move );
				if ( alpha >= beta ) return alpha;
			}
	}

	if ( moves.size() == 0 )
	{
		if (game.board.isCheck( game.turn ))
			return -INFINITY + (needDepth + maxCheckDepth);
		return 0;
	}

	return alpha;
}

int Bioskey(void) 
{
	static int init = 0, pipe;
	static HANDLE inh;
	DWORD dw;
	if (!init) {
		init = 1;
		inh = GetStdHandle(STD_INPUT_HANDLE);
		pipe = !GetConsoleMode(inh, &dw);
		if (!pipe) {
			SetConsoleMode(inh, dw & ~(ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT));
			FlushConsoleInputBuffer(inh);
		}
	}
	if (pipe) {
		if (!PeekNamedPipe(inh, NULL, 0, NULL, &dw, NULL)) return 1;
		return dw;
	}
	else {
		GetNumberOfConsoleInputEvents(inh, &dw);
		return dw <= 1 ? 0 : dw;
	}
}

void checkup()
{
	if ( get_ms() >= stopTime )
	{
		stopSearch = true;
		return;
	}
	if ( Bioskey() )
	{
		string st;
		cin >> st;
		if ( st == "stop" )
			stopSearch = true;
	}
}

int negaScout( int depth, int alpha, int beta )
{
	int ply = needDepth - depth;

	pvLength[ply] = ply;

	if ( depth <= 0 )
	{
		nodes++;
		return forced( alpha, beta, 5 );
	}

	bool firstMove = true;

	list<Move> moves = game.generateMoves( &pvTable[0][ply] );

	//if ( moves.size() == 1 ) depth++; //BUGGY???

	for ( list<Move>::iterator move = moves.begin(); move != moves.end(); move++ )
	{
		if ( nodes & 1023 ) 
			checkup();

		if ( stopSearch ) break;

		int tempScore;

		game.board.makeMove( *move );

		if ( firstMove )
		{
			tempScore = -negaScout( depth - 1, -beta, -alpha );
			firstMove = false;
		}
		else
		{
			tempScore = -negaScout( depth - 1, -alpha - 1, -alpha );
			if ( tempScore > alpha )
			{
				tempScore = -negaScout( depth - 1, -beta, -tempScore );
			}
		}

		game.board.cancelMove( *move );

		if ( tempScore > alpha )
		{
			alpha = tempScore;

			/* update the PV */
			pvTable[ply][ply] = *move;
			for (int j = ply + 1; j < pvLength[ply + 1]; ++j)
				pvTable[ply][j] = pvTable[ply + 1][j];
			pvLength[ply] = pvLength[ply + 1];
		}

		if ( alpha >= beta )
			return beta;
	}

	if ( moves.size() == 0 )
	{
		if (game.board.isCheck( game.turn ))
			return -INFINITY + (needDepth - depth);
		return 0;
	}

	return alpha;
}

string search( int depth )
{
	int startTime = get_ms();

	nodes = 0;
	string bestMove = "ERROR";

	needDepth = depth;
	int ply = needDepth - depth;

	maxScore = -INFINITY;

	bool firstMove = true;

	pvLength[ply] = ply;

	list<Move> moves = game.generateMoves( &pvTable[0][0] );

	int i = 1;

	for ( list<Move>::iterator move = moves.begin(); move != moves.end(); move++ )
	{
		checkup();

		if ( stopSearch ) break;

		int tempScore;

		game.board.makeMove( *move );

		if ( firstMove )
		{
			tempScore = -negaScout( depth - 1, -INFINITY, -maxScore );
			firstMove = false;
		}
		else
		{
			tempScore = -negaScout( depth - 1, -maxScore - 1, -maxScore );
			if ( tempScore > maxScore )
			{
				tempScore = -negaScout( depth - 1, -INFINITY, -maxScore );
			}
		}

		game.board.cancelMove( *move );
		int endTime = get_ms();

		int nps = double(nodes) * 1000 / ( endTime - startTime );

		if ( depth >= 4 )
		{
			cout << "info currmove " << move -> toString() << " currmovenumber " << i << endl;
			cout << "info nps " <<  nps << endl;
		}

		if ( tempScore > maxScore )
		{
			maxScore = tempScore;
			bestMove = move -> toString();

			/* update the PV */
			pvTable[ply][ply] = *move;
			for (int j = ply + 1; j < pvLength[ply + 1]; ++j)
				pvTable[ply][j] = pvTable[ply + 1][j];
			pvLength[ply] = pvLength[ply + 1];
		}

		i++;
	}

	return bestMove;
}

string incrementalSearch( int maxdepth, int time )
{
	string bestmove;
	initPvTable();

	stopTime = get_ms() + time;

	stopSearch = false;

	for ( int depth = 1; depth <= maxdepth; depth++ )
	{
		int startTime = get_ms();
		string tempBestmove = search( depth );
		int endTime = get_ms();

		if ( !stopSearch )
			bestmove = tempBestmove;
		else
			break;

		stringstream nowScore;
		if ( maxScore >= INFINITY - 100 )
			nowScore << "mate " << (INFINITY - maxScore) / 2 + 1;
		else if ( maxScore <= -INFINITY + 100 )
			nowScore << "mate -" << (INFINITY + maxScore) / 2 + 1;
		else
			nowScore << "cp " << maxScore / 10;

		stringstream pvLine;
		for ( int i = 0; i < pvLength[0]; i++ )
		{
			pvLine << " " << pvTable[0][i].toString();
		}

		int nps = double(nodes) * 1000 / ( endTime - startTime );

		cout << "info depth " << depth 
			<< " score "<< nowScore.str() 
			<< " nodes " << nodes 
			<< " time "<< endTime - startTime 
			<< " nps "<< nps
			<< " pv " << pvLine.str() << endl;
	}

	cout << "bestmove " << bestmove << endl;

	return bestmove;
}