#include "uci.h"
#include "game.h"
#include "powder.h"
#include <string>
#include <sstream>
#include "evaluation.h"
#include "perfit.h"
#include "search.h"
#include "constants.h"
#include "pv.h"

void uci() 
{
	cout << "id name " << engineName << endl;
	cout << "id author " << engineAuthor << endl;
	cout << "uciok" << endl;
}

void isready() 
{
	cout << "readyok" << endl;
}

void ucinewgame() 
{
	game.newGame();
}

void position( stringstream& inputStream ) 
{
	string next;
	inputStream >> next;
	if ( next == "startpos" )
		game.newGame();
	if ( next == "fen" )
	{
		string fen, now;
		while ( now != "moves" && !inputStream.eof() )
		{
			inputStream >> now;
			fen += " " + now;
		}
		game.gameFromFen( fen );
		if ( now == "moves" )
		{
			string moves;
			getline( inputStream, moves );
			game.parseMoves( moves );
		}
	}
	inputStream >> next;
	if ( next == "moves" )
	{
		string moves;
		getline( inputStream, moves );
		game.parseMoves( moves );
	}
}

void go( string parameters ) //TODO: Full go
{
	stringstream input;

	input << parameters;

	string first;
	input >> first;

	input >> first;

	int depth = 100;
	int time = 360000;
	if ( first == "depth" )
		input >> depth;
	else if ( first == "movetime" )
		input >> time;
	else if ( first == "infinite" )
		time = 3600000;
	else if ( first == "wtime" || first == "btime" )
	{
		//    .
		string needTurn; 
		
		if ( game.turn == WHITE ) needTurn = "wtime";
		if ( game.turn == BLACK ) needTurn = "btime";

		while ( first != needTurn ) input >> first;

		input >> time;

		if ( game.totalMoves < 40 )
			time /= 40 - game.totalMoves;
		else
			time /= 30;

		cout << time << endl;
	}

	incrementalSearch( depth, time );
}

void quit() 
{
	exit(0);
}

void stop() 
{
	//TODO: STOP
}

void print() 
{
	cout << game.board.toString() << game.turn << endl;
}

using std :: stringstream;

Move lastMove;

void uciStep() 
{
	string input;
	stringstream inputStream;

	getline( cin, input );
	inputStream << input;

	string first;
	inputStream >> first;

	if ( first == "uci" )
		uci();
	else if ( first == "isready" )
		isready();
	else if ( first == "ucinewgame" )
		ucinewgame();
	else if ( first == "position" )
		position( inputStream );
	else if ( first == "go" )
		go( input );
	else if ( first == "mm" )
	{
		string makemove;
		inputStream >> makemove;
		lastMove = Move( makemove );
		game.board.makeMove( lastMove );
	}
	else if ( first == "um" )
	{
		game.board.cancelMove( lastMove );
	}
	else if ( first == "print" )
		print();
	else if ( first == "perfit" )
	{
		int depth;
		inputStream >> depth;

		int startTime = get_ms();
		int perf = perfit( depth );
		int endTime = get_ms();
		
		cout << "perfit result for depth " << depth << " is:" << perf << endl;
		cout << "nodes per sec: " << double(perf) * 1000 / ( endTime - startTime ) << endl;
	}
	else if ( first == "quit" )
		quit();
	else if ( first == "ev" )
	{
		cout << evaluate() << endl;
	}
	else if ( first == "stop" )
		stop();
	else
		cout << "unknown command: " << first << endl;
}

void uciCycle() 
{
	while ( true )
		uciStep();
}